//
//  DOJourneyParametersViewController.m
//  DialogTests
//
//  Created by Christopher Alford on 6/13/13.
//  Copyright (c) 2013 Doonamis. All rights reserved.
//

#import "DOJourneyParametersViewController.h"
#import "DestinationPickerViewController.h"
#import "DateTimePickerViewController.h"
#import "DOMemberViewController.h"

#define STANDARD_CELL_HEIGHT 44.0;
#define PICKER_CELL_HEIGHT 216;
#define MEMBER_CELL_HEIGHT 114;

@interface DOJourneyParametersViewController () <DestinationPickerDelegate, DateTimePickerDelegate, MemberViewDelegate>

@property (weak, nonatomic) IBOutlet UIView *membershipContainer;
@property (weak, nonatomic) IBOutlet UIView *dateContainer;
@property (weak, nonatomic) IBOutlet UIView *destinationContainer;

@property (weak, nonatomic) IBOutlet UILabel *selectedDateLabel;
@property (weak, nonatomic) IBOutlet UILabel *selectedDestinationLabel;

@property (weak, nonatomic) IBOutlet UILabel *womanOnlyLabel;
@property (weak, nonatomic) IBOutlet UISwitch *womanOnlySwitch;
@property (weak, nonatomic) IBOutlet UILabel *cardsLabel;
@property (weak, nonatomic) IBOutlet UISwitch *cardsSwitch;
@property (weak, nonatomic) IBOutlet UIButton *continueButton;
@property (weak, nonatomic) IBOutlet UIButton *cancelButton;

@property (weak, nonatomic) IBOutlet UILabel *memberDetailsTitle;
@property (nonatomic)BOOL memberIsLoggedIn;
@property (nonatomic)BOOL memberDetailIsOpen;
@property (nonatomic)BOOL datePickerIsOpen;
@property (nonatomic)BOOL destinationPickerIsOpen;

@end

@implementation DOJourneyParametersViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Hide the three containers
    self.memberDetailIsOpen = NO;
    self.datePickerIsOpen = NO;
    self.destinationPickerIsOpen = NO;
    
    [self.membershipContainer setHidden:YES];
    [self.dateContainer setHidden:YES];
    [self.destinationContainer setHidden:YES];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    switch (indexPath.section) {
        case 0: {
            if(indexPath.row == 0) {
                
                // Open/Close the member detail if required
                // And update the title depending on their loggedin status
                
                self.memberDetailIsOpen = !self.memberDetailIsOpen;
                
                // Not logged in the title will always be
                if(!self.memberIsLoggedIn) {
                    
                    if(self.memberDetailIsOpen) {
                        self.memberDetailsTitle.text = @"Please enter yor details";
                    }
                    else {
                        self.memberDetailsTitle.text = @"Tap to register";
                    }
                    
                }
                else {
                    
                    if(self.memberDetailIsOpen) {
                        self.memberDetailsTitle.text = @"Please update yor details";
                    }
                    else {
                        self.memberDetailsTitle.text = @"Tap to update your details";
                    }
                    
                }
                
                if(self.memberDetailIsOpen)
                    [self.membershipContainer setHidden:NO];
                else
                    [self.membershipContainer setHidden:YES];

                [self.tableView reloadData];
                break;
            }
        }
            
        case 3:
        case 4: {
            break;
        }
            
        case 1: {
            if(indexPath.row == 0) {
                // Close the destination picker if required
                if(self.destinationPickerIsOpen) {
                    self.destinationPickerIsOpen = NO;
                    [self.destinationContainer setHidden:YES];
                }
                
                // Then open or close dateTimePicker
                if(self.datePickerIsOpen) {
                    self.datePickerIsOpen = NO;
                    [self.dateContainer setHidden:YES];
                    [self.tableView reloadData];
                }
                else {
                    self.datePickerIsOpen = YES;
                    [self.dateContainer setHidden:NO];
                    [self.tableView reloadData];
                }
                
                // Scroll the table view to position the label at the top
                [tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionTop animated:YES];
                
            }
            
            break;
        }
            
        case 2: {
            if(indexPath.row == 0) {
                // Close the dateTime picker if required
                if(self.datePickerIsOpen) {
                    self.datePickerIsOpen = NO;
                    [self.dateContainer setHidden:YES];
                }
                
                // Then open or close the destination picker
                if(self.destinationPickerIsOpen) {
                    self.destinationPickerIsOpen = NO;
                    [self.destinationContainer setHidden:YES];
                    [self.tableView reloadData];
                }
                else {
                    self.destinationPickerIsOpen = YES;
                    [self.destinationContainer setHidden:NO];
                    [self.tableView reloadData];
                }
            }
            
            // Scroll the table view to position the label at the top
            [tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionTop animated:YES];
            break;
        }
    }
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    switch (indexPath.section) {
        case 0: {
            if(indexPath.row == 1) {
                if(self.memberDetailIsOpen) {
                    return MEMBER_CELL_HEIGHT;
                }
                else {
                    return 0;
                }
            }
            break;
        }
            
        case 3:
        case 4: {
            break;
        }
            
        case 1: {
            if(indexPath.row == 1) {
                if(self.datePickerIsOpen) {
                    return PICKER_CELL_HEIGHT;
                }
                else {
                    return 0;
                }
            }
            break;
        }
            
        case 2: {
            if(indexPath.row == 1) {
                if(self.destinationPickerIsOpen) {
                    return PICKER_CELL_HEIGHT;
                }
                else {
                    return 0;
                }
            }
            break;
        }
    }

    return STANDARD_CELL_HEIGHT;
}

- (IBAction)continueTapped:(id)sender {
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if([segue.identifier isEqualToString:@"EmbededDatePickerView"]) {
        DateTimePickerViewController *controller = segue.destinationViewController;
        controller.delegate = self;
    }
    else if([segue.identifier isEqualToString:@"EmbededDestinationPickerView"]) {
        DestinationPickerViewController *controller = segue.destinationViewController;
        controller.delegate = self;
        
    }
    else if([segue.identifier isEqualToString:@"EmbededMemberView"]) {
        DOMemberViewController *controller = segue.destinationViewController;
        controller.delegate = self;
    }
}

#pragma mark - Additional information controls

- (IBAction)womanOnlyChanged:(id)sender {
}

- (IBAction)acceptCardsChanged:(id)sender {
}

#pragma mark - picker delegates

- (void)dateTimeChangedTo:(NSDate *)newDateTime asString:(NSString *)selection
{
    self.selectedDateLabel.text = selection;
    
    /*
    // For reference the formatter corrects the time zone, NSDate is just an instance in time
    // The 12/24 hour format is set by the users preferences

    NSDateFormatter *formatter = [[NSDateFormatter alloc]init];
    formatter.dateFormat = @"dd MMM yy hh:mm:ss a";
    [formatter setTimeStyle:NSDateFormatterFullStyle];
    [formatter setDateStyle:NSDateFormatterMediumStyle];
    NSLog(@"Selected date : %@", [formatter stringFromDate:newDateTime]);
     */
}

- (void)destinationChanged:(NSString *)selection
{
    self.selectedDestinationLabel.text = selection;
}

#pragma mark - MemberView delegate
- (void)memberHasLoggedIn
{
    self.memberIsLoggedIn = YES;
    self.memberDetailIsOpen = NO;
    [self.tableView reloadData];
}

- (void)memberHasLoggedOut
{
    self.memberIsLoggedIn = NO;
    self.memberDetailIsOpen = YES;
    [self.tableView reloadData];
}

- (IBAction)cancelTapped:(id)sender {
    [self dismissViewControllerAnimated:YES completion:nil];
}
@end
