//
//  DOMemberViewController.m
//  DialogTests
//
//  Created by Christopher Alford on 27/06/2013.
//  Copyright (c) 2013 Doonamis. All rights reserved.
//

#import "DOMemberViewController.h"

#import <AddressBook/AddressBook.h>
#import <AddressBookUI/AddressBookUI.h>

#define TF_MOBILE_TAG 100
#define TF_NAME_TAG 101
#define TF_EMAIL_TAG 102

@interface DOMemberViewController () <UITextFieldDelegate, ABPeoplePickerNavigationControllerDelegate>

@property (weak, nonatomic) IBOutlet UITextField *nameInput;
@property (weak, nonatomic) IBOutlet UITextField *emailInput;
@property (weak, nonatomic) IBOutlet UITextField *mobileInput;

@property (strong, nonatomic) NSString *memberName;
@property (strong, nonatomic) NSString *memberEmail;
@property (strong, nonatomic) NSString *memberMobile;

@end

@implementation DOMemberViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - TextField delegate

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event{
    NSLog(@"touchesBegan:withEvent:");
    [self.view endEditing:YES];
    [super touchesBegan:touches withEvent:event];
}

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField{
    NSLog(@"textFieldShouldBeginEditing");
    textField.backgroundColor = [UIColor colorWithRed:.99 green:0.99 blue:0.8 alpha:1.000];
    return YES;
}
- (void)textFieldDidBeginEditing:(UITextField *)textField{
    NSLog(@"textFieldDidBeginEditing");
}

- (BOOL)textFieldShouldEndEditing:(UITextField *)textField{
    NSLog(@"textFieldShouldEndEditing");
    textField.backgroundColor = [UIColor whiteColor];
    return YES;
}
- (void)textFieldDidEndEditing:(UITextField *)textField{
    NSLog(@"textFieldDidEndEditing");
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    // Char by char - test valid email and mobile here?
    if ([string isEqualToString:@"#"]) {
        return NO;
    }
    else {
        return YES;
    }
}

- (BOOL)textFieldShouldClear:(UITextField *)textField{
    NSLog(@"textFieldShouldClear:");
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField{
    
    switch (textField.tag) {
            
        case TF_MOBILE_TAG:
        {
            self.memberMobile = self.mobileInput.text;
            [self.nameInput becomeFirstResponder];
            break;
        }
        case TF_NAME_TAG:
        {
            self.memberName = self.nameInput.text;
            [self.emailInput becomeFirstResponder];
            break;
        }
        case TF_EMAIL_TAG:
        {
            self.memberEmail = self.emailInput.text;
            break;
        }
    }
    [textField resignFirstResponder];
    return YES;
}

#pragma mark - People picker

- (IBAction)showPicker:(id)sender {
    ABPeoplePickerNavigationController *picker = [[ABPeoplePickerNavigationController alloc]init];
    //picker.displayedProperties = @[[NSNumber numberWithInt:kABPersonEmailProperty],[NSNumber numberWithInt:kABPersonCompositeNameFormatFirstNameFirst]];
    picker.peoplePickerDelegate = self;
    
    [self presentViewController:picker animated:YES completion:nil];
}

- (void)peoplePickerNavigationControllerDidCancel:(ABPeoplePickerNavigationController *)peoplePicker
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (BOOL)peoplePickerNavigationController:(ABPeoplePickerNavigationController *)peoplePicker shouldContinueAfterSelectingPerson:(ABRecordRef)person
{
    [self displayPerson:person];
    [self dismissViewControllerAnimated:YES completion:nil];
    return NO;
}

- (BOOL)peoplePickerNavigationController:(ABPeoplePickerNavigationController *)peoplePicker shouldContinueAfterSelectingPerson:(ABRecordRef)person property:(ABPropertyID)property identifier:(ABMultiValueIdentifier)identifier
{
    return NO;
}

- (void)displayPerson:(ABRecordRef)person
{
    NSString *fname = (__bridge_transfer NSString*)ABRecordCopyValue(person, kABPersonFirstNameProperty);
    NSString *lname = (__bridge_transfer NSString*)ABRecordCopyValue(person, kABPersonLastNameProperty);
    
    if(fname && lname)
        self.nameInput.text = [NSString stringWithFormat:@"%@ %@", fname, lname];
    else {
        if(fname)
            self.nameInput.text = fname;
        else
            self.nameInput.text = lname;
    }
    
    NSString *phone = nil;
    NSString *email = nil;
    
    ABMultiValueRef phoneNumbers = ABRecordCopyValue(person,kABPersonPhoneProperty); //kABPersonPhoneIPhoneLabel
    ABMultiValueRef emailAddresses = ABRecordCopyValue(person, kABPersonEmailProperty);
    
    if ((ABMultiValueGetCount(phoneNumbers) > 0) || (ABMultiValueGetCount(emailAddresses) > 0)) {
        
        // Provide adialog where the user can select the correct phone number and email address
        
        phone = (__bridge_transfer NSString *)
        ABMultiValueCopyValueAtIndex(phoneNumbers, 0);
        
        email = (__bridge_transfer NSString *)
        ABMultiValueCopyValueAtIndex(emailAddresses, 0);
    }
    
    self.mobileInput.text = phone;
    self.emailInput.text = email;
    
    CFRelease(phoneNumbers);
    CFRelease(emailAddresses);
}

@end
