//
//  DOViewController.m
//  DialogTests
//
//  Created by Christopher Alford on 6/13/13.
//  Copyright (c) 2013 Doonamis. All rights reserved.
//

#import "DOHomeViewController.h"
#import "MapKit/Mapkit.h"
#import "DOLocationService.h"
#import "SocketsController.h"
#import "SockCtrlPassanger.h"
#import "TAModel.h"
#import "TATaxi.h"
#import "Journey.h"

#import "DOMemberManager.h"

#define ANNOTATION_REGION_PAD_FACTOR 1.25
#define MAX_DEGREES_ARC 30.0
#define MINIMUM_ZOOM_ARC 0.01

typedef enum{
    NO_BOOKINGS,
    BOOKING_PENDING,
    BOOKING_CONFIRMED,
} BookingStatus;

@interface DOHomeViewController () <SockCtrlDelegate, DOLocationServiceDelegate>

@property (weak, nonatomic) IBOutlet MKMapView *mapView;


@property (strong, nonatomic) DOLocationService *locationService;
@property (strong, nonatomic) CLLocation *lastPhysicalLocation;
@property (nonatomic) MKCoordinateRegion currentMapRegion;
@property (strong, nonatomic) Journey *currentJourney;

@property (strong, nonatomic) DOMemberManager *memberManager;
@property (strong, nonatomic) TATaxi *taxi;
@property (nonatomic) BookingStatus bookingStatus;

@property (strong, nonatomic) SockCtrlPassanger *socketsConnection;
@property (weak, nonatomic) IBOutlet UIButton *individualButton;
@property (weak, nonatomic) IBOutlet UIButton *groupButton;
@property (weak, nonatomic) IBOutlet UIButton *miButton;

@end

@implementation DOHomeViewController

- (DOMemberManager *)memberManager
{
    if(!_memberManager) _memberManager = [DOMemberManager sharedInstance];
    return _memberManager;
}

- (SockCtrlPassanger *)socketsConnection
{
    if(!_socketsConnection) {
        _socketsConnection = [SockCtrlPassanger sharedSock];
        _socketsConnection.delegate = self;
    }
    return _socketsConnection;
}

- (DOLocationService *)locationService
{
    if(!_locationService) {
        _locationService = [DOLocationService sharedInstance];
        _locationService.delegate = self;
    }
    return _locationService;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    [self.locationService resumeUpdating];
    
    // Set the initial map region, replace with regionFromCityNamed: later
    self.currentMapRegion = MKCoordinateRegionMakeWithDistance(self.locationService.currentLocation.coordinate, 2500, 2500);
    [self.mapView setRegion:self.currentMapRegion animated:YES];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self startSocket];
    [self.locationService resumeUpdating];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [self stopSocket];
    [self.locationService stopUpdating];
}

- (IBAction)centerLocationTapped:(id)sender {
    if (self.bookingStatus == BOOKING_CONFIRMED && self.taxi) { // Taxi object created and booking confirmed
        [self.mapView showAnnotations:@[self.taxi, self.currentJourney, self.locationService.currentLocation] animated:YES];
    }
    else {
        // Move the map to center on the current physical location
        MKCoordinateRegion region;
        region.center = self.lastPhysicalLocation.coordinate;
        region.span = self.currentMapRegion.span;
        [self.mapView setRegion:region animated:YES];
        self.currentMapRegion = self.mapView.region;
    }
}

- (IBAction)centerButtonTapped:(id)sender {
    [self.individualButton setHidden:!self.individualButton.isHidden];
    [self.groupButton setHidden:!self.groupButton.isHidden];
    [self.miButton setHidden:!self.miButton.isHidden];
}

- (IBAction)individualButtonTapped:(id)sender {
}
- (IBAction)groupButtonTapped:(id)sender {
}
- (IBAction)miButtonTapped:(id)sender {
}

#pragma park - Taxi position socket connections

-(void)startSocket {

    [self.socketsConnection openSocket];
    if (self.locationService.currentLocation) {
        [self.socketsConnection sendPassangerPosition:self.memberManager.memberID
                                             position:self.locationService.currentLocation];
    }
}

-(void)stopSocket {
    [self.socketsConnection closeSocket];
}

#pragma mark - locationService delegate

- (void)newPositionWithLocation:(CLLocation *)newLocation
{
    if(!self.lastPhysicalLocation) {
        self.lastPhysicalLocation = newLocation;
        [self centerLocationTapped:nil];
    }
    else {
        if([newLocation.timestamp timeIntervalSinceDate:self.lastPhysicalLocation.timestamp] > 120 || newLocation.horizontalAccuracy < self.lastPhysicalLocation.horizontalAccuracy)
            self.lastPhysicalLocation = newLocation;
    }
}


@end
