//
//  DateTimePickerViewController.m
//  JoinUpTaxiPassenger
//
//  Created by Christopher Alford on 30/05/2013.
//
//

#import "DateTimePickerViewController.h"

@interface DateTimePickerViewController ()

@property (weak, nonatomic) IBOutlet UIPickerView *picker;

@property (strong, nonatomic) NSMutableArray *dateStrings;
@property (strong, nonatomic) NSMutableArray *hourStrings;
@property (strong, nonatomic) NSMutableArray *minuteStrings;
@property (strong, nonatomic) NSMutableArray *dateValues;

@property (nonatomic, strong) NSDateFormatter *dateFormatter;
@end


@implementation DateTimePickerViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (NSMutableArray *)dateStrings
{
    if(!_dateStrings)_dateStrings = [[NSMutableArray alloc]initWithCapacity:7];
    return _dateStrings;
}

- (NSMutableArray *)hourStrings
{
    if(!_hourStrings) {
        _hourStrings = [[NSMutableArray alloc]init];
        for (NSInteger i = [self.startHour integerValue]; i <= [self.endHour integerValue]; i++) {
            NSString *hour;
            if(i < 10)
                hour = [NSString stringWithFormat:@"0%d", i];
            else
                hour = [NSString stringWithFormat:@"%d", i];
            
            [_hourStrings addObject:hour];
        }
    }
    return _hourStrings;
}

- (NSMutableArray *)minuteStrings
{
    if(!_minuteStrings) {
        _minuteStrings = [[NSMutableArray alloc]init];
        for (NSInteger i = 0; i < 60; i += [self.minuteIncrement integerValue]) {
            NSString *minute;
            if(i < 10)
                minute = [NSString stringWithFormat:@"0%d", i];
            else
                minute = [NSString stringWithFormat:@"%d", i];
            
            [_minuteStrings addObject:minute];
        }
    }
    return _minuteStrings;
}

- (NSMutableArray *)dateValues
{
    if(!_dateValues)_dateValues = [[NSMutableArray alloc]initWithCapacity:7];
    return _dateValues;
}

- (NSNumber *)startHour
{
    if(!_startHour) _startHour = [NSNumber numberWithInt:5];
    return _startHour;
}

- (NSNumber *)endHour
{
    if(!_endHour) _endHour = [NSNumber numberWithInt:23];
    return _endHour;
    
}

- (NSNumber *)minuteIncrement
{
    if(!_minuteIncrement) _minuteIncrement = [NSNumber numberWithInt:15];
    return _minuteIncrement;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.

    self.dateFormatter = [[NSDateFormatter alloc] init];
    [self.dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    [self.dateFormatter setTimeStyle:NSDateFormatterNoStyle];
    
    [self createSevenDays];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Location picker delegate
- (void)pickerView:(UIPickerView *)pickerView didSelectRow: (NSInteger)row inComponent:(NSInteger)component
{
    // Create the string
    NSMutableString *newDateString = [self.dateStrings[[pickerView selectedRowInComponent:0]] mutableCopy];
    [newDateString appendFormat:@" at %@", self.hourStrings[[pickerView selectedRowInComponent:1]]];
    [newDateString appendFormat:@":%@", self.minuteStrings[[pickerView selectedRowInComponent:2]]];
    
    // Add the hours and minutes to the date in the array
    NSDate *newDate = self.dateValues[[pickerView selectedRowInComponent:0]];
    
    NSDateComponents *dayComponent = [[NSDateComponents alloc] init];
    dayComponent.hour = [self.startHour integerValue] + [pickerView selectedRowInComponent:1];
    dayComponent.minute = [self.minuteIncrement integerValue] * [pickerView selectedRowInComponent:2];
    
    NSCalendar *calendar = [NSCalendar currentCalendar];
    
    [self.delegate dateTimeChangedTo:[calendar dateByAddingComponents:dayComponent toDate:newDate options:0] asString:newDateString];
}

// tell the picker how many rows are available for a given component
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component {
    if(component == 0)
        return [self.dateStrings count];
    else if (component == 1)
        return [self.hourStrings count];
    else if (component == 2)
        return [self.minuteStrings count];

    return 0;
}

// tell the picker how many components it will have
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView {
    return 3;
}

// tell the picker the title for a given component
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component {
    if(component == 0)
        return self.dateStrings[row];
    
    else if(component == 1)
        return self.hourStrings[row];
        
    else if(component == 2)
        return self.minuteStrings[row];

    return nil;
}

// tell the picker the width of each row for a given component
- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component {
    if(component == 0)
        return 200.0f;
    else
        return 40.0f;
}

- (void)createSevenDays
{
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"EEE d, MMMM"];
    
    // Populate the dates array with date strings
    // Today plus six days
    NSDate* now = [NSDate date];
    NSDateComponents* tomorrowComponents = [NSDateComponents new] ;
    NSCalendar* calendar = [NSCalendar currentCalendar] ;
    
    for (int i=0; i < 7;i++) {
        tomorrowComponents.day = i;
        NSDate* tomorrow = [calendar dateByAddingComponents:tomorrowComponents toDate:now options:0];
        
        NSDateComponents* tomorrowAt0AMComponents = [calendar components:(NSEraCalendarUnit|NSYearCalendarUnit|NSMonthCalendarUnit|NSDayCalendarUnit) fromDate:tomorrow] ;
        //tomorrowAt0AMComponents.timeZone = [NSTimeZone timeZoneWithName:@"Europe/Budapest"];
        tomorrowAt0AMComponents.hour = 0;
        tomorrowAt0AMComponents.minute = 0;
        tomorrowAt0AMComponents.second = 1;
        NSDate* tomorrowAt0AM = [calendar dateFromComponents:tomorrowAt0AMComponents] ;
        
        if(i==0) {
            [self.dateStrings addObject:NSLocalizedString(@"today", nil)];
        }
        else if(i==1) {
            [self.dateStrings addObject:NSLocalizedString(@"tomorrow", nil)];
        }
        else {
            [self.dateStrings addObject:[dateFormatter stringFromDate:tomorrowAt0AM]];
        }
        [self.dateValues addObject:tomorrowAt0AM];
    }
}



@end
