//
//  DOAddressBookManager.m
//  DialogTests
//
//  Created by Christopher Alford on 08/07/2013.
//  Copyright (c) 2013 Doonamis. All rights reserved.
//

#import "DOAddressBookManager.h"
#import <AddressBook/AddressBook.h>
#import <AddressBook/ABPerson.h>

@interface DOAddressBookManager ()
@property (nonatomic) BOOL accessGranted;
@end

NSString *const kDenied = @"Access to address book is denied";
NSString *const kRestricted = @"Access to address book is restricted";

ABAddressBookRef addressBook;

@implementation DOAddressBookManager

- (id)init
{
    self = [super init];
    if(self) {
        
        return self;
    }
    return nil;
}

- (void)authorize
{
    CFErrorRef error = NULL;
    
    switch (ABAddressBookGetAuthorizationStatus()){
        case kABAuthorizationStatusAuthorized:
        {
            addressBook = ABAddressBookCreateWithOptions(NULL, &error);
            self.accessGranted = YES;
            break;
        }
            
        case kABAuthorizationStatusDenied:
        {
            self.accessGranted = NO;
            [self displayMessage:kDenied];
            if (addressBook != NULL) {
                CFRelease(addressBook);
            }
            break;
        }
            
        case kABAuthorizationStatusNotDetermined:
        {
            addressBook = ABAddressBookCreateWithOptions(NULL, &error);
            ABAddressBookRequestAccessWithCompletion
            (addressBook, ^(bool granted, CFErrorRef error) {
                if (granted){
                    self.accessGranted = YES;
                }
                else {
                    self.accessGranted = NO;
                }
                if (addressBook != NULL){
                    CFRelease(addressBook);
                }
            });
            break;
        }
            
        case kABAuthorizationStatusRestricted:
        {
            self.accessGranted = NO;
            [self displayMessage:kRestricted];
            break;
        }
    }
}

- (NSArray *)doesPersonExistWithFullName:(NSString *)paramFullName
{
    [self authorize];
    
    if (addressBook == NULL){
        NSLog(@"Address book is null.");
        return NO;
    }
    
    NSMutableArray *contacts;
    
    CFArrayRef allPeopleWithThisName = ABAddressBookCopyPeopleWithName(addressBook, (__bridge CFStringRef)paramFullName);
    
    contacts = [[NSMutableArray alloc]init];
        
    for(int i = 0; i < [allPeopleWithThisName count]; i++) {
            ABRecordRef record = allPeopleWithThisName[i];
    }
        
        if (addressBook != NULL){
            CFRelease(addressBook);
        }
        return allPeopleWithThisName;


    return nil;
}


- (BOOL) doesPersonExistWithFirstName:(NSString *)paramFirstName
                             lastName:(NSString *)paramLastName
                        inAddressBook:(ABRecordRef)paramAddressBook
{
    if(!self.accessGranted)
        [self authorize];
    BOOL result = NO;
    
    if (paramAddressBook == NULL){
        NSLog(@"The address book is null.");
        return NO;
    }
    
    NSArray *allPeople = (__bridge_transfer NSArray *)
    ABAddressBookCopyArrayOfAllPeople(paramAddressBook);
    
    NSUInteger peopleCounter = 0;
    for (peopleCounter = 0; peopleCounter < [allPeople count]; peopleCounter++){
        
        ABRecordRef person = (__bridge ABRecordRef)
        allPeople[peopleCounter];
        
        NSString *firstName = (__bridge_transfer NSString *) ABRecordCopyValue(person, kABPersonFirstNameProperty);
        
        NSString *lastName = (__bridge_transfer NSString *) ABRecordCopyValue(person, kABPersonLastNameProperty);
        
        BOOL firstNameIsEqual = NO;
        BOOL lastNameIsEqual = NO;
        
        if ([firstName length] == 0 && [paramFirstName length] == 0){
            firstNameIsEqual = YES;
        }
        else if ([firstName isEqualToString:paramFirstName]){
            firstNameIsEqual = YES;
        }
        
        if ([lastName length] == 0 && [paramLastName length] == 0){
            lastNameIsEqual = YES;
        }
        else if ([lastName isEqualToString:paramLastName]){
            lastNameIsEqual = YES;
        }
        
        if (firstNameIsEqual &&
            lastNameIsEqual){
            return YES;
        }
    }

    return result;
}

- (BOOL) doesGroupExistWithGroupName:(NSString *)paramGroupName
                       inAddressBook:(ABAddressBookRef)paramAddressBook{
    [self authorize];
    BOOL result = NO;
    
    if (paramAddressBook == NULL){
        NSLog(@"The address book is null.");
        return NO;
    }
    
    NSArray *allGroups = (__bridge_transfer NSArray *)
    ABAddressBookCopyArrayOfAllGroups(paramAddressBook);
    
    NSUInteger groupCounter = 0;
    for (groupCounter = 0;
         groupCounter < [allGroups count];
         groupCounter++){
        
        ABRecordRef group = (__bridge ABRecordRef) [allGroups objectAtIndex:groupCounter];
        
        NSString *groupName = (__bridge_transfer NSString *) ABRecordCopyValue(group, kABGroupNameProperty);
        
        if ([groupName length] == 0 && [paramGroupName length] == 0){
            return YES;
        }
        
        else if ([groupName isEqualToString:paramGroupName]){
            return YES;
        }
        
    }
    
    if (addressBook != NULL){
        CFRelease(addressBook);
    }
    
    return result;
    
}

- (void) displayMessage:(NSString *)paramMessage{
    [[[UIAlertView alloc] initWithTitle:nil
                                message:paramMessage
                               delegate:nil
                      cancelButtonTitle:@"OK"
                      otherButtonTitles:nil] show];
}

@end
