//
//  TAModel.m
//  JoinUpTaxiTaxi
//
//  Created by Ferran Falguera on 18/06/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "TAModel.h"
#import "SVProgressHUD.h"
#import "SockCtrlPassanger.h"
#import "TADestination.h"
#import "TACityDestination.h"
#import "TAMember.h"
#import "SockCtrlPassenger.h"
#import "SocketsController.h"

@interface TAModel () <So
@property (strong, nonatomic) TAMember *member;
- (NSURL *)applicationDocumentsDirectory;
@end

@implementation TAModel

@synthesize storeCoordinator = __persistentStoreCoordinator;
@synthesize managedObjectModel = __managedObjectModel;
@synthesize managedObjectContext = __managedObjectContext;

@synthesize scheduledJourneys, pendingVote, currentJourney, sentenceCO2, currentServiceETA;

static TAModel *sharedModel = nil;

+ (TAModel*)sharedModel {
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedModel = [[TAModel alloc] init];
    });
    return sharedModel;
}

- (TAMember *)member
{
    if(!_member) _member = [TAMember shared];
    return _member;
}

-(void) fetchLogin  {
    
    // Find any matches
    NSEntityDescription *loginEntity = [NSEntityDescription entityForName:@"Login" inManagedObjectContext:self.managedObjectContext];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:loginEntity];
    //[request setPredicate: [NSPredicate predicateWithFormat: @"type == %@",type]];
    NSError *error;
    NSArray *results = [self.managedObjectContext executeFetchRequest:request error:&error];
    
    if([results count] > 0){  // Login exists
        self.login = results[0];
        TAMember *member = [TAMember shared];
        member.memberID = self.login.idPassenger;
    }
    else{  // No match found so create a new Login entity
        self.login = (Login *)[NSEntityDescription
                                      insertNewObjectForEntityForName:@"Login"
                                      inManagedObjectContext:self.managedObjectContext];
    }
    // Save the entity
    [self.managedObjectContext save:&error];
    if (error != nil) {
        NSLog(@"TAModel error creating Login object%@", [error localizedDescription]);
    }
}

-(void) removeLogin {
    // Borrem de la BBDD
    [__managedObjectContext deleteObject:self.login];
    // Commit
    [self saveContext];
    // Carreguem de la BBDD perquè el posi a nul
    [self fetchLogin];
    self.member = nil;

}


- (id)init {
    if (self = [super init]) {
        // Inicialització local
        didRegisterToken = NO;
    }
    return self;
}

-(void)loadContext {
    [self fetchLogin];
}

- (void)saveContext
{
    [[SockCtrlPassanger sharedSock] closeSocket];
    [[SockCtrlPassanger sharedSock] openSocket];
    NSError *error = nil;
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil)
    {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error])
        {
            /*
             Replace this implementation with code to handle the error appropriately.
             
             abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. 
             */
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        } 
    }
}

#pragma mark - Core Data stack

/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (__persistentStoreCoordinator != nil)
    {
        return __persistentStoreCoordinator;
    }
    
    NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"Model.sqlite"];
    
    NSError *error = nil;
    __persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![__persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:nil error:&error])
    {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. 
         
         Typical reasons for an error here include:
         * The persistent store is not accessible;
         * The schema for the persistent store is incompatible with current managed object model.
         Check the error message to determine what the actual problem was.
         
         
         If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
         
         If you encounter schema incompatibility errors during development, you can reduce their frequency by:
         * Simply deleting the existing store:
         [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
         
         * Performing automatic lightweight migration by passing the following dictionary as the options parameter: 
         [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption, [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
         
         Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
         
         */
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }    
    
    return __persistentStoreCoordinator;
}

/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *)managedObjectContext
{
    if (__managedObjectContext != nil)
    {
        return __managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil)
    {
        __managedObjectContext = [[NSManagedObjectContext alloc] init];
        [__managedObjectContext setPersistentStoreCoordinator:coordinator];
    }
    return __managedObjectContext;
}

/**
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created from the application's model.
 */
- (NSManagedObjectModel *)managedObjectModel
{
    if (__managedObjectModel != nil)
    {
        return __managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"Model" withExtension:@"momd"];
    __managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return __managedObjectModel;
}



#pragma mark - Application's Documents directory

/**
 Returns the URL to the application's Documents directory.
 */
- (NSURL *)applicationDocumentsDirectory
{
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}


- (void)reLoginSilently:(bool)silent {
    //if (opLogin) [opLogin cancel];
    if ([self.login.idPassenger intValue]!= 0) {
        _opLogin = [[TAOpLoginPas alloc] initWithPassengerID:self.login.idPassenger
                                                      silent:silent
                                                    delegate:self];
    }
}

-(NSArray*)cityDestinationsStr {
    NSMutableArray *mutArr = [[NSMutableArray alloc] initWithCapacity:_cityDestinations.count];
    for (TACityDestination *city in _cityDestinations) {
        [mutArr addObject:city.cityName];
    }
    return mutArr;
}

-(NSString*) cityStr4Origin:(CLLocationCoordinate2D) coordinate {
    CLLocation *loc = [[CLLocation alloc] initWithLatitude:coordinate.latitude longitude:coordinate.longitude];
    for (TACityDestination *city in _cityDestinations) {
        CLLocation *locCity = [[CLLocation alloc] initWithLatitude:city.coordinate.latitude longitude:city.coordinate.longitude];
        if (city.radius > [loc distanceFromLocation:locCity]) {
            return city.cityName;
        }
    }
    // Tornem la primera de la llista
    NSString *cityName = ((TACityDestination*)_cityDestinations[0]).cityName;
    return cityName;
}

-(NSArray*)destinationsInCity:(NSString*)city {
    
    // Modified to return all the destination data not just the name
    TACityDestination *tmpCity = [[TACityDestination alloc] init];
    tmpCity.cityName = city;
    int idxCity = [_cityDestinations indexOfObject:tmpCity];
    if (idxCity == NSNotFound) idxCity = 0;
    TACityDestination *cityDestination = [_cityDestinations objectAtIndex:idxCity];

    return cityDestination.destinations;
    /*
    NSMutableArray *mutArr = [NSMutableArray new];
    for (TADestination *destination in cityDestination.destinations) {
        [mutArr addObject:destination.name];
    }
    return mutArr;
    */
}
/*
-(NSArray*)destinationsStr {
    NSMutableArray *mutArr = [[NSMutableArray new] autorelease];
    for (TADestination *destination in destinations) {
        [mutArr addObject:destination.name];
    }
    return mutArr;
}
*/
- (void)reLogDestinations {
    if (_lastGetDestinations == nil || [_lastGetDestinations timeIntervalSinceNow] < -60*60*24) {
        opGetDestinations = [[TAOpGetDestinationsWithCity alloc] initWithDelegate:self];
    }
}

-(void) setToken:(NSString *)token {
    _token = token;
    [self registerPushServerIfPossible];
}

-(NSString*) token {
    return _token;
}

-(void)registerPushServerIfPossible {
    if (!didRegisterToken && _token && [self.login.idPassenger intValue] != 0) {
        didRegisterToken = YES;
        NSString *path = [[NSBundle mainBundle] pathForResource:@"config" ofType:@"plist"];
        NSDictionary *config = [[NSDictionary alloc] initWithContentsOfFile:path];
        
        NSMutableString *regToken = [[NSMutableString alloc] initWithString:[config objectForKey:@"BasePUSHURL"]];
        
        [regToken appendFormat:@"operation=register&userId=%@&pushId=%@&platform=ios", self.login.idPassenger, _token];
        // Register the Device Data
        NSURL *url = [[NSURL alloc] initWithString:regToken];
        NSURLRequest *request = [[NSURLRequest alloc] initWithURL:url];
        NSData *returnData = [NSURLConnection sendSynchronousRequest:request returningResponse:nil error:nil];
        NSString *returnString = [[NSString alloc] initWithData:returnData encoding:NSUTF8StringEncoding];
        NSLog(@"Return Data: %@", returnString);
    }
}

-(void)didFinishCall:(NSObject *)call withResult:(FinishedRes)ok {
    if (call == _opLogin) {
        TAMember *member = [TAMember shared];
        member.loginSet = YES;

        [self registerPushServerIfPossible];
    }
}

@end
