//
//  BrewersFetch.m
//  BeerFest
//
//  Created by Christopher Alford on 12/02/2013.
//  Copyright (c) 2013 GNO. All rights reserved.
//

#import "BrewersFetch.h"
#import "Brewers.h"
#import "AppDelegate.h"

@interface BrewersFetch()
@property (strong, nonatomic) NSMutableData *receivedData;
@property (strong, nonatomic) NSManagedObjectContext *context;
@end

@implementation BrewersFetch

- (NSManagedObjectContext *)context
{
    AppDelegate *appdel = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    return [appdel managedObjectContext];
}

- (void)processJSONOnjects:(id)jsonObjects
{
    
    if([jsonObjects isKindOfClass:[NSDictionary class]]){
        NSArray *results = [NSArray arrayWithArray:[jsonObjects objectForKey:@"results"]];
        for(NSDictionary *dict in results){
            // Nothing to do
        }
    }
    else if([jsonObjects isKindOfClass:[NSArray class]] && ([jsonObjects count] > 0)){
        
        // As this is only a demo
        [self clearStoreOfEntitiesNamed:@"Brewers"];
        
        for(NSDictionary *dict in jsonObjects){
            [self storeItem:dict];
        }
        [[NSNotificationCenter defaultCenter] postNotificationName: @"NewBrewers" object:self];
    }
}


/***********************************
 / storeItem
 /
 / Test each JSON item for the correct data type,
 / if is correct and is not NULL save value,
 / or nil if it's a NULL string, newline, etc.
 /
 **********************************/
- (void)storeItem:(NSDictionary *)entry{
    
    Brewers *entity = (Brewers *)[NSEntityDescription
                                                 insertNewObjectForEntityForName:@"Brewers"
                                                 inManagedObjectContext:self.context];
    
    id brewerID = [entry objectForKey:@"id"];
    if([brewerID isKindOfClass:[NSString class]])
    {
        NSNumberFormatter * f = [[NSNumberFormatter alloc] init];
        [f setNumberStyle:NSNumberFormatterDecimalStyle];
        [entity setBrewerID:[f numberFromString:brewerID]];
    }
    else {
        return;
    }
    
    id thumbURL = [entry objectForKey:@"thumb_url"];
    if([thumbURL isKindOfClass:[NSString class]]) {
        [entity setThumbURL:thumbURL];
    }
    else {
        return;
    }
    
    id brewerURL = [entry objectForKey:@"brewer_url"];
    if([brewerURL isKindOfClass:[NSString class]]) {
        [entity setBrewerURL:brewerURL];
    }
    else {
        return;
    }
    
    id brewerName = [entry objectForKey:@"name"];
    if([brewerName isKindOfClass:[NSString class]]) {
        [entity setBrewerName:brewerName];
    }
    else {
        return;
    }
    
    id brewerLocation = [entry objectForKey:@"location_en"];
    if([brewerLocation isKindOfClass:[NSString class]]) {
        [entity setBrewerLocation:brewerLocation];
    }
    else {
        return;
    }
    
    // Save the new entity
    NSError *error = nil;
    if(![self.context save:&error]) {
        NSLog(@"CatalogueFetch::storeItem %@", [error localizedDescription]);
    }
}

- (void)dumpEntity:(Brewers *)entity{
    NSLog(@"thumbURL        :%@", entity.thumbURL);
    NSLog(@"brewerURL       :%@",entity.brewerURL);
    NSLog(@"brewerName      :%@",entity.brewerName);
    NSLog(@"brewerLocation  :%@",entity.brewerLocation);
}

- (void)dumpStoreWithEntityNamed:(NSString *)entityName
{
    NSEntityDescription *entityDescription = [NSEntityDescription entityForName:entityName
                                                         inManagedObjectContext:self.context];
    
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entityDescription];
    
    NSError *error = nil;
    
    NSArray *results = [NSArray arrayWithArray:[self.context executeFetchRequest:request error:&error]];
    for(Brewers *b in results){
        [self dumpEntity:b];
    }
    
    if(error){
        NSLog(@"%@Fetch::dumpStore : %@",entityName, [error localizedDescription]);
    }
}



@end
