//
//  JSONFetch.m
//  GekkoBooks
//
//  Created by Christopher Alford on 17/02/2013.
//  Copyright (c) 2013 GNO. All rights reserved.
//

#import "JSONFetch.h"

#import "DOAppDelegate.h"

@interface JSONFetch()

@property (nonatomic, strong)NSMutableData *receivedData;
@property (nonatomic, strong)NSURLConnection *itemConnection;
@property (nonatomic, strong)NSManagedObjectContext *context;

@property (strong, nonatomic)NSDictionary *config;
@property (strong, nonatomic)NSString *base;

@end

@implementation JSONFetch

- (id)initWithBaseURL:(NSString *)baseURL
{
    self = [super init];
    if(self) {
        NSString *path = [[NSBundle mainBundle] pathForResource:@"config" ofType:@"plist"];
        _config = [[NSDictionary alloc] initWithContentsOfFile:path];
        _base = [_config valueForKey:baseURL];
        return self;
    }
    return nil;
}

- (NSManagedObjectContext *)context
{
    DOAppDelegate *appdel = (DOAppDelegate *)[[UIApplication sharedApplication] delegate];
    return [appdel managedObjectContext];
}

- (void)getItems
{
    // If we haven't been given a date get all available items
    // Boilerplate to allow for items added, deleted, modified
    // Could use match, insert, update routines
    
    NSString *fullURLString;
    if(self.base) {
        fullURLString = [NSString stringWithFormat:@"%@%@", self.base, self.urlString];
    }
    else {
        fullURLString = self.urlString;
    }
    
    NSURL *url = [NSURL URLWithString:fullURLString];
    
    NSURLRequest *theRequest=[NSURLRequest requestWithURL:url
                                              cachePolicy:NSURLRequestUseProtocolCachePolicy
                                          timeoutInterval:60.0];
    
    self.itemConnection = [[NSURLConnection alloc] initWithRequest:theRequest delegate:self];
    
    if(self.itemConnection){
        [self.delegate fetchStarted];
        [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
        self.receivedData = [[NSMutableData alloc]init];
    }
    
}

#pragma mark NSURLConnectionDelegate methods

- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge {
    [self.delegate fetchReportedMessage:[NSString stringWithFormat:@"JSONFetch Connection didReceiveAuthenticationChallenge: %@", challenge ]];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    NSHTTPURLResponse* httpResponse = (NSHTTPURLResponse*)response;
    NSInteger statusCode = [httpResponse statusCode];
    [self.delegate fetchReportedMessage:[NSString stringWithFormat:@"JSONFetch Connection didReceiveResponse: Status : %d - MimeType : %@",statusCode, [response MIMEType]]];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [self.receivedData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    [self.delegate fetchFailedWithError:error];
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
    NSLog(@"JSONFetch Connection didFailWithError %@",[error localizedDescription]);
    
    UIAlertView *alertView = [[UIAlertView alloc]
                              initWithTitle:NSLocalizedString(@"NetworkErrorTitle", @"")
                              message:NSLocalizedString(@"NetworkErrorMsg", @"")
                              delegate:self
                              cancelButtonTitle:NSLocalizedString(@"OkButton", @"")
                              otherButtonTitles:nil];
    
    [alertView show];
    [connection cancel];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    [self.delegate fetchCompleted];
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
    id jsonObjects;
    
    // We are using iOS 5 opnwards so no need to support SBJsonPaser
    // But left in here to demonstrate
    if(NSClassFromString(@"NSJSONSerialization")) {
        // iOS 5 JSON Parse
        NSError *error;
        jsonObjects = [NSJSONSerialization JSONObjectWithData:self.receivedData
                                                      options:kNilOptions
                                                        error:&error];
        if(error) {
            NSLog(@"%@", [error localizedDescription]);
            return;
        }
        [self processJSONOnjects:jsonObjects];
    }
}

- (void)processJSONOnjects:(id)jsonObjects
{
}

#pragma mark - Core Data methods

- (void)clearStoreOfEntitiesNamed:(NSString *)entityName {
    
    NSLog(@"JSONFetch : Clearing %@", entityName);

    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:[NSEntityDescription entityForName:entityName inManagedObjectContext:self.context]];
    [request setIncludesPropertyValues:NO]; //only fetch the managedObjectID
    
    NSError *error = nil;
    NSArray *items = [self.context executeFetchRequest:request error:&error];
    
    for(NSManagedObject *item in items) {
        [self.context deleteObject:item];
    }
    
    if(![self.context save:&error]) {
        NSLog(@"JSONFetch -> clearStoreWithEntityName:%@: %@",entityName, [error localizedDescription]);
    }
}

- (void)storeItem:(NSDictionary *)dictionary{
    
}

- (void)dumpEntity:(NSManagedObject *)object{
    
}

- (void)dumpStoreWithEntityNamed:(NSString *)entityName{
    
}



@end
