//
//  SockCtrl.m
//  JoinUpTaxis
//
//  Created by Ferran Falguera on 14/05/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SocketsController.h"

@interface SocketsController () <NSStreamDelegate>

@property (strong, nonatomic) NSTimer *keepAliveTimer;
@property (strong, nonatomic) NSMutableString *receivedData;
@property (nonatomic) BOOL alreadyConnected;

@end

@implementation SocketsController



-(id) init {
    self = [super init];
    if(self)
        return self;
    return nil;
}

- (NSMutableString *)receivedData
{
    if (!_receivedData) _receivedData = [[NSMutableString alloc]init];
    return _receivedData;
}

- (void) initNetworkCommunication {
    	
	CFReadStreamRef readStream;
	CFWriteStreamRef writeStream;
    NSString *path = [[NSBundle mainBundle] pathForResource:@"config" ofType:@"plist"];
    NSDictionary *config = [[NSDictionary alloc] initWithContentsOfFile:path];
    
    NSString *ip = [config objectForKey:@"RTServerIP"];
	
	CFStreamCreatePairWithSocketToHost(NULL, (__bridge CFStringRef)ip, 55555, &readStream, &writeStream);
	
	self.inputStream = (__bridge NSInputStream *)readStream;
	self.outputStream = (__bridge NSOutputStream *)writeStream;
    
	[self.inputStream setDelegate:self];
	[self.outputStream setDelegate:self];
    
	[self.inputStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
	[self.outputStream scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    
    [self.receivedData setString:@""];
	[self.inputStream open];
	[self.outputStream open];
}

#pragma mark Tractaments Interns

-(void)insertIntoReceivedData:(NSInputStream *)stream {
    // Recollim totes les dades rebudes fins al moment
    uint8_t buffer[1024];
    int len;
    while ([stream hasBytesAvailable]) {
        len = [stream read:buffer maxLength:sizeof(buffer)];
        if (len > 0) {
            [self.receivedData appendString:[[NSString alloc] initWithBytes:buffer
                                                            length:len
                                                          encoding:NSUTF8StringEncoding]];
        }
    }
    NSLog(@"%@", self.receivedData);
}

- (NSArray *)chunkReceivedPackets {
    // Iterem sobre les lectures separades per el \n
    unsigned endOfChunk = 0;
    NSMutableArray* chunksArray = [[NSMutableArray alloc] init];
    while ([self.receivedData length] != 0) {
        endOfChunk = [[self.receivedData substringFromIndex:0] 
                      rangeOfString:@"\n"].location;
        
        if (NSNotFound == endOfChunk) {
            //Tenim trama incomplerta emmagatzemada en mutReceived
            break;
        }
        
        NSString* theSubstring = [NSString stringWithString:[self.receivedData substringWithRange:NSMakeRange(0,endOfChunk)]];
        [chunksArray addObject:theSubstring];
        // endOfChunk+1 perquè hem de saltar el separador \n
        [self.receivedData setString:[self.receivedData substringFromIndex:endOfChunk+1]];
    }
    return chunksArray;
}

#pragma mark Control Socket

- (void)stream:(NSStream *)theStream handleEvent:(NSStreamEvent)streamEvent {
    
	NSLog(@"stream event %i", streamEvent);
	
	switch (streamEvent) {
			
		case NSStreamEventOpenCompleted:
			NSLog(@"Stream opened");
            self.alreadyConnected = YES;
			break;
            
		case NSStreamEventHasBytesAvailable:
            NSLog(@"Socket: NSStreamEventHasBytesAvailable");
            if (theStream == self.inputStream) {
                // Recollim tota la informació en un mutable array
                [self insertIntoReceivedData: self.inputStream];
                
                // Separem les trames que ens han arribat
                NSArray *chunksArray = [self chunkReceivedPackets];
                
                // Iterem sobre les trames en cru, separant les estructures JSON
                // cridant al delegate per al seu tractament
                for (NSString *chunk in chunksArray) {
                    
                    NSError *error;
                    id jsonObjects = [NSJSONSerialization JSONObjectWithData:[chunk dataUsingEncoding:NSUTF8StringEncoding]
                                                                  options:kNilOptions
                                                                    error:&error];
                    if(error) {
                        NSLog(@"%@", [error localizedDescription]);
                        return;
                    }

                   if (self.delegate) {
                        [self.delegate receivedData:jsonObjects];
                    }
                }
			}
			break;
            
		case NSStreamEventErrorOccurred:
			NSLog(@"Socket error: %@", [theStream streamError]);
            [theStream close];
            [theStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            if (theStream == self.inputStream) {
                self.inputStream = nil;
            }
            if (theStream == self.outputStream) {
                self.outputStream = nil;
            }
			break;
			
		case NSStreamEventEndEncountered:
            NSLog(@"Socket: NSStreamEventEndEncountered");
            [theStream close];
            [theStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
            theStream = nil;
			
			break;
            
        case NSStreamEventNone:
            NSLog(@"Socket: NSStreamEventNone");
            break;
        
        case NSStreamEventHasSpaceAvailable:
            if (self.alreadyConnected) {
                self.alreadyConnected = NO;
                if (self.positionToSendWhenConnected) {
                    [self.outputStream write:[self.positionToSendWhenConnected bytes] maxLength:[self.positionToSendWhenConnected length]];
                }
                [self manageEventHasSpaceAvailable];
            }
            break;
            
		default:
			NSLog(@"Unknown event");
	}
}

-(void)manageEventHasSpaceAvailable {
    //Funció hook per a les subclasses
}

- (NSData *)JSONDictionaryToNSData:(NSDictionary *)json {
    NSError *error = nil;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:json
                                                       options:NSJSONWritingPrettyPrinted
                                                         error:&error];
    if(error) {
        NSLog(@"Error creating NSData from JSONDictionary : %@", [error localizedDescription]);
        return nil;
    }
    return jsonData;
}


#pragma mark Funcions Publiques

-(void)sendKeepAlive {
    if ([self.outputStream streamStatus] == NSStreamStatusOpen) {
        NSDictionary *keepAlive = [NSDictionary dictionaryWithObjectsAndKeys:
                                   @"KeepAlive",@"type", nil];
        NSData *data = [self JSONDictionaryToNSData:keepAlive];
        [self.outputStream write:[data bytes] maxLength:[data length]];
    }
}

- (void)onControlLoop:(NSTimer*)theTimer {
    // Si no tenim els streams habilitats inicialitzem comunicacions
    if (!self.outputStream) {
        [self initNetworkCommunication];
    } else if ([self.outputStream streamStatus] == NSStreamStatusOpen) {
        [self sendKeepAlive];
    }
}

-(void)openSocket {
   self.keepAliveTimer = [NSTimer scheduledTimerWithTimeInterval:5.0
                                                          target:self
                                                        selector:@selector(onControlLoop:)
                                                        userInfo:nil
                                                         repeats:YES];
}

-(void)closeSocket {
    // Parem el timer
    [self.keepAliveTimer invalidate];
    self.keepAliveTimer = nil;
    
    // alliberem el socket
    if (self.outputStream.streamStatus == NSStreamStatusOpen) {
        [self.outputStream close];
        self.outputStream = nil;
    }
    if (self.inputStream.streamStatus == NSStreamStatusOpen) {
        [self.inputStream close];
        self.inputStream = nil;
    }
    [self.outputStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
    [self.inputStream removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
}

@end
