//
//  TAOpGetDestinations.m
//  JoinUpTaxiPassenger
//
//  Created by Ferran Falguera on 17/07/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "TAOpGetDestinationsWithCity.h"
#import "TADestination.h"
#import "TACityDestination.h"
#import "TAModel.h"

#import <CoreData/CoreData.h>
#import "TAModel.h"
#import "City.h"
#import "Destination.h"

@interface TAOpGetDestinationsWithCity ()

@property (strong, nonatomic) NSManagedObjectContext *context;

@end

@implementation TAOpGetDestinationsWithCity

- (NSManagedObjectContext *)context{
    if(!_context) {
        TAModel *model = [TAModel sharedModel];
        _context = [model managedObjectContext];
    }
    return _context;
}

-(id)initWithDelegate:(id<TAHTTPCallDelegate>)delegate {
    
    NSString *get =  [NSString stringWithFormat:@"op=GetDestinationsWithCity"];
    
    return [super initWithQuery:get loadingMsg:nil successMsg:nil body:nil  baseURLConfigKey:BaseJSONURL delegate:delegate];
    
}

- (void)processContainingJSON:(NSDictionary *)json {
    
    if(json) {
        TAModel *model = [TAModel sharedModel];
        [model deleteCities];

    }
            
    NSMutableArray *cityArr = [[NSMutableArray alloc] init];
    
    for (NSDictionary *cityDict in [json objectForKey:@"Cities"]) {
        [self newCityEntityWithDictionary:cityDict];
    }
    
    // Sort the cities by cityName
    NSSortDescriptor *valueDescriptor = [[NSSortDescriptor alloc] initWithKey:@"cityName" ascending:YES];
    NSArray * descriptors = [NSArray arrayWithObject:valueDescriptor];
    NSArray * citySorted = [cityArr sortedArrayUsingDescriptors:descriptors];

    // Keep them in your model object
    [TAModel sharedModel].cityDestinations = citySorted;
}

- (void)newCityEntityWithDictionary:(NSDictionary *)cityDict
{
    City *cityEntity = (City *)[NSEntityDescription insertNewObjectForEntityForName:@"City"
                                                             inManagedObjectContext:self.context];
    id idCity = [cityDict objectForKey:@"IdCity"];
    if([idCity isKindOfClass:[NSNumber class]]) {
        cityEntity.cityID = idCity;
        
        id cityName = [cityDict objectForKey:@"Name"];
        if([cityName isKindOfClass:[NSString class]])
            cityEntity.name = cityName;
            
        id latitude = [cityDict objectForKey:@"CityLat"];
        if([latitude isKindOfClass:[NSNumber class]])
            cityEntity.latitude = [NSNumber numberWithDouble:[latitude doubleValue] / 1000000];
            
        id longitude = [cityDict objectForKey:@"CityLon"];
        if([longitude isKindOfClass:[NSNumber class]])
            cityEntity.longitude = [NSNumber numberWithDouble:[longitude doubleValue] / 1000000];
            
        id radius = [cityDict objectForKey:@"Radius"];
        if([radius isKindOfClass:[NSNumber class]])
            cityEntity.radius = radius;
        
        // Capture the destination objects
        id destinations = [cityDict objectForKey:@"Destinations"];
        if([destinations isKindOfClass:[NSArray class]]) {
            for(NSDictionary *destinationDictionary in destinations) {
    
                Destination *destinationEntity = (Destination *)[NSEntityDescription insertNewObjectForEntityForName:@"Destination"
                                                                                              inManagedObjectContext:self.context];
                id idDestination = [destinationDictionary objectForKey:@"IdDestination"];
                if([idDestination isKindOfClass:[NSNumber class]])
                    destinationEntity.destinationID = idDestination;
                
                id name = [destinationDictionary objectForKey:@"DestinationName"];
                if([name isKindOfClass:[NSString class]])
                    destinationEntity.name = name;
                
                id latitude = [destinationDictionary objectForKey:@"DestinationLat"];
                if([latitude isKindOfClass:[NSNumber class]])
                    destinationEntity.latitude = [NSNumber numberWithDouble:[latitude doubleValue] / 1000000];
                
                id longitude = [destinationDictionary objectForKey:@"DestinationLon"];
                if([longitude isKindOfClass:[NSNumber class]])
                    destinationEntity.longitude = [NSNumber numberWithDouble:[longitude doubleValue] / 1000000];
                
                [cityEntity addHasDestinationsObject:destinationEntity];
            }
        }
        NSError *error;
        if(![self.context save:&error])
        {
            NSLog(@"Error saving City entity : %@", [error localizedDescription]);
        }
    }
    else {
        NSLog(@">>>> Error processing City entity, not a NSNumber!");
    }
}


@end
